//----------------------------------------------------------------------------
// C main line
//----------------------------------------------------------------------------

#include <m8c.h>        // part specific constants and macros
#include "PSoCAPI.h"    // PSoC API definitions for all User Modules

/* --- global value */
WORD g_int;

/* --- I2C buffer */
static BYTE g_rcvBuffer[16];
static BYTE rcvCommand[16];
static BYTE g_sendBuffer[4];

/* ------------------------------------- */
/* --- PWM1 interrupt (10.92msec) */
#pragma interrupt_handler Timer1Int
void Timer1Int(void)
{
	g_sendBuffer[0] = PRT2DR ^ 0xff;
	if (g_int > 0) {
		g_int -= 1;
	}
}

void Timer1Wait(WORD time) {
	g_int = time;
	while(g_int) {}
}

/* ------------------------------------- */
/* --- LED  */
void led_set(BYTE port, BYTE level)
{
	BYTE s, rs;
	
	s = 0b00000001 << port;
	rs = s ^ 0xff;
	if (level == 255) {
		//PRT0DM0 |= s;
		//PRT0DM1 |= s;
		//PRT0DM2 &= rs;
		PRT0DR &= rs;
		PRT0GS &= rs;
	}
	else if (level == 0) {
		//PRT0DM0 |= s;
		//PRT0DM1 |= s;
		//PRT0DM2 &= rs;
		PRT0DR |= s;
		PRT0GS &= rs;
	}
	else {
		level = 254 - level;
		switch(port) {
			case 0:
				PWM8_1_WritePulseWidth(level);
				break;
				
			case 1:
				PWM8_2_WritePulseWidth(level);
				break;
				
			case 2:
				PWM8_3_WritePulseWidth(level);
				break;
				
			case 3:
				PWM8_4_WritePulseWidth(level);
				break;
				
			case 4:
				PWM8_5_WritePulseWidth(level);
				break;
				
			case 5:
				PWM8_6_WritePulseWidth(level);
				break;
				
			case 6:
				PWM8_7_WritePulseWidth(level);
				break;
				
			case 7:
				PWM8_8_WritePulseWidth(level);
				break;
		}
		PRT0GS |= s;
	}
}

void led_controller(BYTE *buf, BYTE size)
{
	char in, i;
	
	if (size == 1) {
		in = buf[0];
		for(i=0; i<8; i++) {
			if (in & 1) {
				buf[i+1] = 255;
			}
			in >>= 1;
		}
	}

	in = buf[0];
	for(i=0; i<8; i++) {
		if ((in & 1) == 0) {
			buf[i+1] = 0;
		}
		led_set(i, buf[i+1]);
		in >>= 1;
	}
}


/* ------------------------------------- */
/* --- main loop */
void main(void)
{
	BYTE status, size;

    /* --- initialize */
	PWM8_1_Start();
	PWM8_2_Start();
	PWM8_3_Start();
	PWM8_4_Start();
	PWM8_5_Start();
	PWM8_6_Start();
	PWM8_7_Start();
	PWM8_8_Start();
	PWM8_1_EnableInt();
	I2CHW_1_Start();
	I2CHW_1_EnableSlave();
	I2CHW_1_InitWrite(g_rcvBuffer, 16);
	I2CHW_1_InitRamRead(g_sendBuffer, 1);
	I2CHW_1_EnableInt();
	M8C_EnableGInt;
	
	PRT2DR = 0b11111111;
	
	led_set(0, 255);
	led_set(1, 255);
	led_set(2, 255);
	led_set(3, 255);
	led_set(4, 255);
	led_set(5, 255);
	led_set(6, 255);
	led_set(7, 255);

	while(1) {
		status = I2CHW_1_bReadI2CStatus();
		/* --- receive data */
		if ( status & I2CHW_WR_COMPLETE ) {
			size = I2CHW_1_Write_Count;
			rcvCommand[0] = g_rcvBuffer[0];
			rcvCommand[1] = g_rcvBuffer[1];
			rcvCommand[2] = g_rcvBuffer[2];
			rcvCommand[3] = g_rcvBuffer[3];
			rcvCommand[4] = g_rcvBuffer[4];
			rcvCommand[5] = g_rcvBuffer[5];
			rcvCommand[6] = g_rcvBuffer[6];
			rcvCommand[7] = g_rcvBuffer[7];
			rcvCommand[8] = g_rcvBuffer[8];
			I2CHW_1_ClrWrStatus();
			I2CHW_1_InitWrite(g_rcvBuffer, 16);
			/* receive work */
			led_controller(rcvCommand, size);
		}
		if ( status & I2CHW_WR_OVERFLOW ) {
			I2CHW_1_ClrWrStatus();
			I2CHW_1_InitWrite(g_rcvBuffer, 8);		
		}
		/* --- send data */
		if ( status & I2CHW_RD_COMPLETE ) {
			I2CHW_1_ClrRdStatus();  
	        I2CHW_1_InitRamRead(g_sendBuffer, 4);
		}
	}

}
